import { utilities as nestWinstonModuleUtilities } from 'nest-winston';
import * as winston from 'winston';
const environment = process.env.NODE_ENV || 'local';

export const winstonLoggerOptions: winston.LoggerOptions = {
  level:
    environment === 'local'
      ? 'debug'
      : environment === 'development'
        ? 'warn'
        : 'error',
  transports: [
    new winston.transports.Console({
      level:
        environment === 'local'
          ? 'debug'
          : environment === 'development'
            ? 'warn'
            : 'error',
      format: winston.format.combine(
        winston.format.timestamp(),
        nestWinstonModuleUtilities.format.nestLike('MyApp', {
          prettyPrint: environment === 'local',
        }),
      ),
    }),
    // Only create file logs if NOT local
    ...(environment !== 'local'
      ? [
          new winston.transports.File({
            filename: 'logs/error.log',
            level: 'error',
            format: winston.format.json(),
          }),
          new winston.transports.File({
            filename: 'logs/combined.log',
            level: 'info',
            format: winston.format.json(),
          }),
        ]
      : []),
  ],
};
