import { Column, Entity, JoinColumn, ManyToOne, Unique } from 'typeorm';
import { SceneDeviceDto } from '../dtos';
import { AbstractEntity } from '../../abstract/entities/abstract.entity';
import { SceneSwitchesTypeEnum } from '@app/common/constants/scene-switch-type.enum';
import { DeviceEntity } from '../../device/entities';
import { SceneEntity } from '../../scene/entities';

@Entity({ name: 'scene-device' })
@Unique(['device', 'switchName'])
export class SceneDeviceEntity extends AbstractEntity<SceneDeviceDto> {
  @Column({
    type: 'uuid',
    default: () => 'gen_random_uuid()',
    nullable: false,
  })
  public uuid: string;

  @ManyToOne(() => DeviceEntity, (device) => device.sceneDevices, {
    nullable: false,
  })
  @JoinColumn({ name: 'device_uuid' })
  device: DeviceEntity;

  @Column({
    nullable: false,
  })
  sceneUuid: string;

  @Column({
    nullable: false,
    type: 'enum',
    enum: SceneSwitchesTypeEnum,
  })
  switchName: SceneSwitchesTypeEnum;

  @Column({
    nullable: false,
  })
  automationTuyaUuid: string;

  @ManyToOne(() => SceneEntity, (scene) => scene.sceneDevices, {
    nullable: false,
  })
  @JoinColumn({ name: 'scene_uuid' })
  scene: SceneEntity;

  @Column({
    nullable: false,
    default: false,
  })
  public disabled: boolean;

  constructor(partial: Partial<SceneDeviceEntity>) {
    super();
    Object.assign(this, partial);
  }
}
