import { AbstractEntity } from '@app/common/modules/abstract/entities/abstract.entity';
import { ProductEntity } from '@app/common/modules/product/entities';
import { SubspaceModelProductAllocationEntity } from '@app/common/modules/space-model';
import { NewTagEntity } from '@app/common/modules/tag/entities/tag.entity';
import { Column, Entity, ManyToOne, Unique } from 'typeorm';
import { SubspaceProductAllocationDto } from '../../dtos/subspace-product-allocation.dto';
import { SubspaceEntity } from './subspace.entity';

@Entity({ name: 'subspace_product_allocation' })
@Unique(['subspace', 'product', 'tag'])
export class SubspaceProductAllocationEntity extends AbstractEntity<SubspaceProductAllocationDto> {
  @Column({
    type: 'uuid',
    default: () => 'gen_random_uuid()',
    nullable: false,
  })
  public uuid: string;

  @ManyToOne(() => SubspaceEntity, (subspace) => subspace.productAllocations, {
    nullable: false,
    onDelete: 'CASCADE',
  })
  public subspace: SubspaceEntity;

  @ManyToOne(() => SubspaceModelProductAllocationEntity, {
    nullable: true,
    onDelete: 'SET NULL',
  })
  public inheritedFromModel?: SubspaceModelProductAllocationEntity;

  @ManyToOne(() => ProductEntity, { nullable: false, onDelete: 'CASCADE' })
  public product: ProductEntity;

  @ManyToOne(() => NewTagEntity, { nullable: true, onDelete: 'CASCADE' })
  public tag: NewTagEntity;

  constructor(partial: Partial<SubspaceProductAllocationEntity>) {
    super();
    Object.assign(this, partial);
  }
}
