import { TimeZoneRepository } from '@app/common/modules/timezone/repositories';
import { allTimeZones } from '@app/common/constants/timezones';
import { Injectable } from '@nestjs/common';

@Injectable()
export class TimeZoneSeeder {
  constructor(private readonly timeZoneRepository: TimeZoneRepository) {}

  async addTimeZoneDataIfNotFound(): Promise<void> {
    try {
      const existingTimeZones = await this.timeZoneRepository.find();
      const timeZoneNames = existingTimeZones.map((tz) => tz.cityName);

      const missingTimeZones = allTimeZones.filter(
        (tz) => !timeZoneNames.includes(tz.name),
      );

      if (missingTimeZones.length > 0) {
        await this.addTimeZoneData(missingTimeZones);
      }
    } catch (err) {
      console.error('Error while checking time zone data:', err);
      throw err;
    }
  }

  private async addTimeZoneData(
    timeZones: { name: string; offset: string }[],
  ): Promise<void> {
    try {
      const timeZoneEntities = timeZones.map((tz) => ({
        cityName: tz.name,
        timeZoneOffset: tz.offset,
      }));

      await this.timeZoneRepository.save(timeZoneEntities);
    } catch (err) {
      console.error('Error while adding time zone data:', err);
      throw err;
    }
  }
}
