-- This model gives the average hourly set and current temperatures per space, per device
-- The only issue witht this model is that it does not represent 24 hours/device. which is normal when  no changelog is being recorded.
--Shall I fill the missing hours 
WITH avg_set_temp AS (-- average set temperature per device per hour
    SELECT 
        device.uuid AS device_id,
        device.space_device_uuid AS space_id,
        event_time::date AS date,
        DATE_PART('hour', event_time) AS hour,
        AVG("device-status-log".value::INTEGER) AS avg_set_temp
    FROM device
    LEFT JOIN "device-status-log"
        ON device.uuid = "device-status-log".device_id
    LEFT JOIN product
        ON product.uuid = device.product_device_uuid
    WHERE product.name = 'Smart Thermostat'
    AND "device-status-log".code = 'temp_set'
    GROUP BY 1,2,3,4
)

, avg_current_temp as (
	SELECT 
        device.uuid AS device_id,
        device.space_device_uuid AS space_id,
        event_time::date AS date,
        DATE_PART('hour', event_time) AS hour,
        AVG("device-status-log".value::INTEGER) AS avg_current_temp
    FROM device
    LEFT JOIN "device-status-log"
        ON device.uuid = "device-status-log".device_id
    LEFT JOIN product
        ON product.uuid = device.product_device_uuid
    WHERE product.name = 'Smart Thermostat'
    AND "device-status-log".code = 'temp_current'
    GROUP BY 1,2,3,4
)

, joined_data AS ( -- this will return null values for hours where there was no previously set temperature
    SELECT 
        current_temp.device_id,
        current_temp.space_id, 
        current_temp.date,
        current_temp.hour,
        set_temp.avg_set_temp,
        current_temp.avg_current_temp,
        ROW_NUMBER() OVER (PARTITION BY current_temp.device_id, current_temp.space_id ORDER BY current_temp.date, current_temp.hour) AS row_num
    FROM avg_current_temp AS current_temp
    LEFT JOIN avg_set_temp AS set_temp
        ON set_temp.device_id = current_temp.device_id
        AND set_temp.space_id = current_temp.space_id
        AND set_temp.date = current_temp.date
        AND set_temp.hour = current_temp.hour
)

, filled_data AS (
    SELECT 
        a.device_id,
        a.space_id,
        a.date,
        a.hour,
        COALESCE(
            a.avg_set_temp,
            (SELECT b.avg_set_temp 
             FROM joined_data b 
             WHERE b.device_id = a.device_id 
               AND b.space_id = a.space_id 
               AND b.row_num < a.row_num 
               AND b.avg_set_temp IS NOT NULL 
             ORDER BY b.row_num DESC 
             LIMIT 1)
        ) AS avg_set_temp,
        a.avg_current_temp
    FROM joined_data a
)

SELECT *
FROM filled_data
ORDER BY 1,3,4;