type TypeORMIncludeQuery = string[];

const mappingInclude: { [key: string]: any } = {
  roles: {
    role: true,
  },
  users: {
    user: true,
  },
  community: {
    community: true,
  },
  space: {
    space: true,
  },
  subspace: {
    subspace: true,
  },
  project: {
    project: true,
  },
  'space-model': {
    subspaceModels: 'subspace-model',
    tags: 'tag_model',
  },
  'subspace-model': {
    tags: 'tag_model',
  },
  tag_model: {
    tag_model: true,
    product: true,
  },
};

export function buildTypeORMIncludeQuery(
  modelName: string,
  includeParam?: string,
): TypeORMIncludeQuery | undefined {
  if (includeParam) {
    const relations: TypeORMIncludeQuery = [];
    const fieldsToInclude: string[] = includeParam.split(',');

    fieldsToInclude.forEach((field: string) => {
      const nestedFields = field.split('.');
      let currentModelName = modelName;
      let isValid = true;

      nestedFields.forEach((nestedField, index) => {
        const currentMapping = mappingInclude[currentModelName];
        if (currentMapping?.[nestedField]) {
          currentModelName =
            typeof currentMapping[nestedField] === 'string'
              ? currentMapping[nestedField]
              : currentModelName;
        } else {
          console.warn(
            `Field "${nestedFields.slice(0, index + 1).join('.')}" not found in mappingInclude for model "${currentModelName}"`,
          );
          isValid = false;
          return;
        }
      });

      if (isValid) {
        relations.push(field);
      }
    });

    return relations.length ? relations : undefined;
  }

  return undefined;
}
