import {
  Body,
  Controller,
  Get,
  HttpStatus,
  Param,
  Post,
  Put,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiTags, ApiOperation } from '@nestjs/swagger';
import { UserNotificationService } from '../services/user-notification.service';
import {
  UserNotificationAddDto,
  UserNotificationUpdateDto,
} from '../dtos/user-notification.dto';

import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { ControllerRoute } from '@app/common/constants/controller-route';

@ApiTags('User Notification Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.USER_NOTIFICATION.ROUTE,
})
export class UserNotificationController {
  constructor(
    private readonly userNotificationService: UserNotificationService,
  ) {}

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post()
  @ApiOperation({
    summary: ControllerRoute.USER_NOTIFICATION.ACTIONS.ADD_SUBSCRIPTION_SUMMARY,
    description:
      ControllerRoute.USER_NOTIFICATION.ACTIONS.ADD_SUBSCRIPTION_DESCRIPTION,
  })
  async addUserSubscription(
    @Body() userNotificationAddDto: UserNotificationAddDto,
  ) {
    const addDetails = await this.userNotificationService.addUserSubscription(
      userNotificationAddDto,
    );
    return {
      statusCode: HttpStatus.CREATED,
      message: 'User Notification Added Successfully',
      data: addDetails,
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get(':userUuid')
  @ApiOperation({
    summary:
      ControllerRoute.USER_NOTIFICATION.ACTIONS.FETCH_SUBSCRIPTIONS_SUMMARY,
    description:
      ControllerRoute.USER_NOTIFICATION.ACTIONS.FETCH_SUBSCRIPTIONS_DESCRIPTION,
  })
  async fetchUserSubscriptions(@Param('userUuid') userUuid: string) {
    const userDetails =
      await this.userNotificationService.fetchUserSubscriptions(userUuid);
    return {
      statusCode: HttpStatus.OK,
      message: 'User Notification fetched Successfully',
      data: { ...userDetails },
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put()
  @ApiOperation({
    summary:
      ControllerRoute.USER_NOTIFICATION.ACTIONS.UPDATE_SUBSCRIPTION_SUMMARY,
    description:
      ControllerRoute.USER_NOTIFICATION.ACTIONS.UPDATE_SUBSCRIPTION_DESCRIPTION,
  })
  async updateUserSubscription(
    @Body() userNotificationUpdateDto: UserNotificationUpdateDto,
  ) {
    await this.userNotificationService.updateUserSubscription(
      userNotificationUpdateDto,
    );
    return {
      statusCode: HttpStatus.OK,
      message: 'User subscription updated Successfully',
    };
  }
}
