import { ApiProperty } from '@nestjs/swagger';
import {
  IsNotEmpty,
  IsString,
  IsArray,
  ValidateNested,
  IsEnum,
  Length,
  IsOptional,
  IsEmail,
} from 'class-validator';
import { Type } from 'class-transformer';
import { WorkingDays } from '@app/common/constants/working-days';
import { VisitorPasswordEnum } from '@app/common/constants/visitor-password.enum';

class ScheduleDto {
  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;

  @ApiProperty({
    description: 'workingDay',
    enum: WorkingDays,
    isArray: true,
    required: true,
  })
  @IsArray()
  @IsEnum(WorkingDays, { each: true })
  @IsNotEmpty()
  public workingDay: WorkingDays[];
}

export class AddDoorLockOnlineMultipleDto {
  @ApiProperty({
    description: 'email',
    required: true,
  })
  @IsEmail()
  @IsNotEmpty()
  public email: string;
  @ApiProperty({
    description: 'devicesUuid',
    required: true,
  })
  @IsArray()
  @IsNotEmpty()
  public devicesUuid: [string];

  @ApiProperty({
    description: 'password name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public passwordName: string;
  @ApiProperty({
    description: 'password',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  @Length(7, 7)
  public password: string;

  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;

  @ApiProperty({
    description: 'scheduleList',
    type: [ScheduleDto],
    required: false,
  })
  @IsArray()
  @ValidateNested({ each: true })
  @IsOptional()
  @Type(() => ScheduleDto)
  public scheduleList: ScheduleDto[];
}
export class AddDoorLockOnlineOneTimeDto {
  @ApiProperty({
    description: 'email',
    required: true,
  })
  @IsEmail()
  @IsNotEmpty()
  public email: string;
  @ApiProperty({
    description: 'devicesUuid',
    required: true,
  })
  @IsArray()
  @IsNotEmpty()
  public devicesUuid: [string];

  @ApiProperty({
    description: 'password name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public passwordName: string;
  @ApiProperty({
    description: 'password',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  @Length(7, 7)
  public password: string;

  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;
}
export class AddDoorLockOfflineOneTimeDto {
  @ApiProperty({
    description: 'email',
    required: true,
  })
  @IsEmail()
  @IsNotEmpty()
  public email: string;
  @ApiProperty({
    description: 'password name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public passwordName: string;
  @ApiProperty({
    description: 'devicesUuid',
    required: true,
  })
  @IsArray()
  @IsNotEmpty()
  public devicesUuid: [string];
}
export class AddDoorLockOfflineMultipleDto {
  @ApiProperty({
    description: 'email',
    required: true,
  })
  @IsEmail()
  @IsNotEmpty()
  public email: string;
  @ApiProperty({
    description: 'devicesUuid',
    required: true,
  })
  @IsArray()
  @IsNotEmpty()
  public devicesUuid: [string];

  @ApiProperty({
    description: 'password name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public passwordName: string;

  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;
}
export class AddDoorLockTemporaryPasswordDto {
  @ApiProperty({
    description: 'Operation type',
    enum: VisitorPasswordEnum,
    required: true,
  })
  @IsEnum(VisitorPasswordEnum)
  @IsNotEmpty()
  public operationType: VisitorPasswordEnum;

  @ApiProperty({
    description: 'email',
    required: true,
  })
  @IsEmail()
  @IsNotEmpty()
  public email: string;

  @ApiProperty({
    description: 'devicesUuid',
    required: true,
  })
  @IsArray()
  @IsNotEmpty()
  public devicesUuid: string[];

  @ApiProperty({
    description: 'password name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public passwordName: string;

  @ApiProperty({
    description: 'password',
    required: false,
  })
  @IsString()
  @IsOptional()
  @Length(7, 7)
  public password: string;

  @ApiProperty({
    description: 'effectiveTime',
    required: false,
  })
  @IsString()
  @IsOptional()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: false,
  })
  @IsString()
  @IsOptional()
  public invalidTime: string;

  @ApiProperty({
    description: 'scheduleList',
    type: [ScheduleDto],
    required: false,
  })
  @IsArray()
  @ValidateNested({ each: true })
  @IsOptional()
  @Type(() => ScheduleDto)
  public scheduleList: ScheduleDto[];
}
